# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

import unittest

from sql import Flavor


class TestFlavor(unittest.TestCase):

    def test(self):
        Flavor()

    def test_limitstyle(self):
        flavor = Flavor(limitstyle='rownum')

        self.assertEqual(flavor.limitstyle, 'rownum')

    def test_invalid_limitstyle(self):
        with self.assertRaises(ValueError):
            Flavor(limitstyle='foo')

    def test_max_limit(self):
        flavor = Flavor(max_limit=42)

        self.assertEqual(flavor.max_limit, 42)

    def test_invalid_max_limit(self):
        with self.assertRaises(ValueError):
            Flavor(max_limit='foo')

    def test_paramstyle_format(self):
        flavor = Flavor(paramstyle='format')

        self.assertEqual(flavor.paramstyle, 'format')
        self.assertEqual(flavor.param, '%s')

    def test_paramstyle_qmark(self):
        flavor = Flavor(paramstyle='qmark')

        self.assertEqual(flavor.paramstyle, 'qmark')
        self.assertEqual(flavor.param, '?')

    def test_invalid_paramstyle(self):
        with self.assertRaises(ValueError):
            Flavor(paramstyle='foo')



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import Cast, Column, Table


class TestCast(unittest.TestCase):
    column = Column(Table('t'), 'c')

    def test_cast(self):
        for cast in [Cast(self.column, 'int'), self.column.cast('int')]:
            self.assertEqual(str(cast), 'CAST("c" AS int)')
            self.assertEqual(cast.params, ())

    def test_cast_no_expression(self):
        cast = Cast(1.1, 'int')
        self.assertEqual(str(cast), 'CAST(%s AS int)')
        self.assertEqual(cast.params, (1.1,))



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import doctest
import os

import sql

here = os.path.dirname(__file__)
readme = os.path.normpath(os.path.join(here, '..', '..', 'README.rst'))


def load_tests(loader, tests, pattern):
    tests.addTests(loader.discover(start_dir=here, pattern=pattern))
    for mod in (sql,):
        tests.addTest(doctest.DocTestSuite(mod))
    if os.path.isfile(readme):
        tests.addTest(doctest.DocFileSuite(
                readme, module_relative=False,
                tearDown=lambda t: sql.Flavor.set(sql.Flavor())))
    return tests



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import For, Table


class TestFor(unittest.TestCase):
    def test_for(self):
        for_ = For('UPDATE', Table('t1'), Table('t2'), nowait=True)
        self.assertEqual(str(for_), 'FOR UPDATE OF "t1", "t2" NOWAIT')

    def test_for_single_table(self):
        for_ = For('UPDATE')
        for_.tables = Table('t1')
        self.assertEqual(str(for_), 'FOR UPDATE OF "t1"')

    def test_invalid_type(self):
        with self.assertRaises(ValueError):
            For('foo')



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import Collate, Column, Table


class TestCollate(unittest.TestCase):
    column = Column(Table('t'), 'c')

    def test_collate(self):
        for collate in [Collate(self.column, 'C'), self.column.collate('C')]:
            self.assertEqual(str(collate), '"c" COLLATE "C"')
            self.assertEqual(collate.params, ())

    def test_collate_no_expression(self):
        collate = Collate("foo", 'C')
        self.assertEqual(str(collate), '%s COLLATE "C"')
        self.assertEqual(collate.params, ("foo",))



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

import unittest

from sql import Excluded


class TestExcluded(unittest.TestCase):

    def test_alias(self):
        self.assertEqual(Excluded.alias, 'EXCLUDED')

    def test_has_alias(self):
        self.assertFalse(Excluded.has_alias)



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import threading
import unittest

from sql import AliasManager, Table


class TestAliasManager(unittest.TestCase):

    def setUp(self):
        self.synchro = threading.Event()
        self.succeed1 = threading.Event()
        self.succeed2 = threading.Event()
        self.finish1 = threading.Event()
        self.finish2 = threading.Event()

        self.t1 = Table('t1')
        self.t2 = Table('t2')

    def func1(self):
        try:
            with AliasManager(exclude=[self.t2]):
                a1 = AliasManager.get(self.t1)
                a2 = AliasManager.get(self.t2)
                self.synchro.wait()
                self.assertEqual(a1, AliasManager.get(self.t1))
                self.assertEqual(a2, AliasManager.get(self.t2))
                self.succeed1.set()
            return
        except Exception:
            pass
        finally:
            self.finish1.set()

    def func2(self):
        try:
            with AliasManager(exclude=[self.t2]):
                a2 = AliasManager.get(self.t2)
                a1 = AliasManager.get(self.t1)
                self.synchro.set()
                self.assertEqual(a1, AliasManager.get(self.t1))
                self.assertEqual(a2, AliasManager.get(self.t2))
                self.succeed2.set()
            return
        except Exception:
            pass
        finally:
            self.synchro.set()
            self.finish2.set()

    def test_threading(self):

        th1 = threading.Thread(target=self.func1)
        th2 = threading.Thread(target=self.func2)

        th1.start()
        th2.start()

        self.finish1.wait()
        self.finish2.wait()
        if not self.succeed1.is_set() or not self.succeed2.is_set():
            self.fail()

    def test_contains(self):
        with AliasManager():
            AliasManager.get(self.t1)
            self.assertTrue(AliasManager.contains(self.t1))

    def test_contains_exclude(self):
        with AliasManager(exclude=[self.t1]):
            self.assertEqual(AliasManager.get(self.t1), '')
            self.assertFalse(AliasManager.contains(self.t1))

    def test_set(self):
        with AliasManager():
            AliasManager.set(self.t1, 'foo')
            self.assertEqual(AliasManager.get(self.t1), 'foo')



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import Table


class TestTable(unittest.TestCase):

    def test_name(self):
        t = Table('mytable')
        self.assertEqual(str(t), '"mytable"')

    def test_quoted_name(self):
        t = Table('my "quoted" name')
        self.assertEqual(str(t), '"my ""quoted"" name"')

    def test_schema(self):
        t = Table('mytable', schema='myschema')
        self.assertEqual(str(t), '"myschema"."mytable"')

    def test_database(self):
        t = Table('mytable', database='mydatabase', schema='myschema')
        self.assertEqual(str(t), '"mydatabase"."myschema"."mytable"')



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

import unittest

from sql import Rollup


class TestRollup(unittest.TestCase):

    def test_invalid_expressions(self):
        with self.assertRaises(ValueError):
            Rollup('foo')



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import CombiningQuery, Table, Union, With


class TestUnion(unittest.TestCase):
    q1 = Table('t1').select()
    q2 = Table('t2').select()
    q3 = Table('t3').select()

    def test_invalid_queries(self):
        with self.assertRaises(ValueError):
            CombiningQuery('foo', 'bar')

    def test_union2(self):
        query = Union(self.q1, self.q2)
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b"')
        self.assertEqual(tuple(query.params), ())

        query = self.q1 | self.q2
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b"')
        self.assertEqual(tuple(query.params), ())

    def test_union_with(self):
        table = Table('t')
        with_ = With()
        with_.query = table.select(table.id, where=table.id == 1)
        query = Union(self.q1, self.q2, with_=with_)

        self.assertEqual(str(query),
            'WITH "a" AS ('
            'SELECT "b"."id" FROM "t" AS "b" WHERE ("b"."id" = %s)) '
            'SELECT * FROM "t1" AS "c" UNION SELECT * FROM "t2" AS "d"')
        self.assertEqual(tuple(query.params), (1,))

    def test_union3(self):
        query = Union(self.q1, self.q2, self.q3)
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b" '
            'UNION SELECT * FROM "t3" AS "c"')
        self.assertEqual(tuple(query.params), ())

        query = Union(Union(self.q1, self.q2), self.q3)
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b" '
            'UNION SELECT * FROM "t3" AS "c"')
        self.assertEqual(tuple(query.params), ())

        query = Union(self.q1, Union(self.q2, self.q3))
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b" '
            'UNION SELECT * FROM "t3" AS "c"')
        self.assertEqual(tuple(query.params), ())

        query = self.q1 | self.q2 | self.q3
        self.assertEqual(str(query),
            'SELECT * FROM "t1" AS "a" UNION SELECT * FROM "t2" AS "b" '
            'UNION SELECT * FROM "t3" AS "c"')
        self.assertEqual(tuple(query.params), ())



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest

from sql import Table
from sql.conditionals import Case, Coalesce, Greatest, Least, NullIf


class TestConditionals(unittest.TestCase):
    table = Table('t')

    def test_case(self):
        case = Case((self.table.c1, 'foo'),
            (self.table.c2, 'bar'),
            else_=self.table.c3)
        self.assertEqual(str(case),
            'CASE WHEN "c1" THEN %s '
            'WHEN "c2" THEN %s '
            'ELSE "c3" END')
        self.assertEqual(case.params, ('foo', 'bar'))

    def test_case_no_expression(self):
        case = Case((True, self.table.c1), (self.table.c2, False),
            else_=False)
        self.assertEqual(str(case),
            'CASE WHEN %s THEN "c1" '
            'WHEN "c2" THEN %s '
            'ELSE %s END')
        self.assertEqual(case.params, (True, False, False))

    def test_case_sql(self):
        case = Case(
            (self.table.select(self.table.bool,
                    where=self.table.c2 == 'bar'), self.table.c1),
            else_=self.table.select(self.table.c1,
                where=self.table.c2 == 'foo'))
        self.assertEqual(str(case),
            'CASE WHEN '
            '(SELECT "a"."bool" FROM "t" AS "a" WHERE ("a"."c2" = %s)) '
            'THEN "c1" '
            'ELSE (SELECT "a"."c1" FROM "t" AS "a" WHERE ("a"."c2" = %s)) END')
        self.assertEqual(case.params, ('bar', 'foo'))

    def test_coalesce(self):
        coalesce = Coalesce(self.table.c1, self.table.c2, 'foo')
        self.assertEqual(str(coalesce), 'COALESCE("c1", "c2", %s)')
        self.assertEqual(coalesce.params, ('foo',))

    def test_coalesce_sql(self):
        coalesce = Coalesce(
            self.table.select(self.table.c1, where=self.table.c2 == 'bar'),
            self.table.c2)
        self.assertEqual(str(coalesce),
            'COALESCE('
            '(SELECT "a"."c1" FROM "t" AS "a" WHERE ("a"."c2" = %s)), "c2")')
        self.assertEqual(coalesce.params, ('bar',))

    def test_nullif(self):
        nullif = NullIf(self.table.c1, 'foo')
        self.assertEqual(str(nullif), 'NULLIF("c1", %s)')
        self.assertEqual(nullif.params, ('foo',))

    def test_greatest(self):
        greatest = Greatest(self.table.c1, self.table.c2, 'foo')
        self.assertEqual(str(greatest), 'GREATEST("c1", "c2", %s)')
        self.assertEqual(greatest.params, ('foo',))

    def test_least(self):
        least = Least(self.table.c1, self.table.c2, 'foo')
        self.assertEqual(str(least), 'LEAST("c1", "c2", %s)')
        self.assertEqual(least.params, ('foo',))



# This file is part of python-sql.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

import unittest

from sql import CombiningQuery, From, Table


class TestFrom(unittest.TestCase):

    def test_add(self):
        t1 = Table('t1')
        t2 = Table('t2')
        from_ = From([t1]) + t2

        self.assertEqual(from_, [t1, t2])

    def test_invalid_add(self):
        with self.assertRaises(TypeError):
            From([Table('t')]) + 'foo'

    def test_invalid_add_combining_query(self):
        with self.assertRaises(TypeError):
            From([Table('t')]) + CombiningQuery(
                Table('t1').select(), Table('t2').select())
